/*
 * LoggerFactory.java
 *
 * Created on 16 May 2009
 *
 * Version 1.1
 *
 * Copyright (c) 2009 Kieran Greer
 *
 */

package org.jlog2;


import org.jlog2.util.*;

import java.util.*;


/**
 * This abstract class must be implemented to return logger instances for each specific logger type.
 * @author Kieran Greer
 */
public abstract class LoggerFactory
{
    /**
     * Default log channels to be added to every logger. Of type LogChannel.
     */
    protected Vector defaultLogChannels;

    /**
     * Default debug channels to be added to every logger. Of type LogChannel.
     */
    protected Vector defaultDebugChannels;

    /**
     * The log channels mapped to the appropriate logger ids. Keys are the logger
     * ids of type String and values are the log channels of type LogChannels.
     */
    protected Hashtable logChannels;

    /**
     * The debug channels mapped to the appropriate logger ids. Keys are the logger
     * ids of type String and values are the debug channels of type LogChannels.
     */
    protected Hashtable debugChannels;
    
    /** The highest level to log messages at */
    private static String logLevel;

    /** The highest level to log debug errors at */
    private static String debugLevel;


    /**
     * Create a new instance of LoggerFactory.
     * @throws java.lang.Exception any error.
     */
    public LoggerFactory() throws Exception
    {
        initialiseValues();
    }
    
    /**
     * Initialise some values.
     * @throws java.lang.Exception any error.
     */
    private void initialiseValues() throws Exception
    {
        PropertiesParser propsParser;               //properties parser

        defaultLogChannels = new Vector();
        defaultDebugChannels = new Vector();
        logChannels = new Hashtable();
        debugChannels = new Hashtable();

        //load properties and default channels
        propsParser = new PropertiesParser();
        propsParser.initialiseChannels(logChannels, debugChannels);
        if (propsParser.getProperties().containsKey(LogConst.LOGFILELEVEL))
        {
            logLevel = propsParser.getProperties().getProperty(LogConst.LOGFILELEVEL);
        }
        else
        {
            logLevel = LoggerHandler.INFO;
        }

        if (propsParser.getProperties().containsKey(LogConst.DEBUGFILELEVEL))
        {
            debugLevel = propsParser.getProperties().getProperty(LogConst.DEBUGFILELEVEL);
        }
        else
        {
            debugLevel = LoggerHandler.INFO;
        }
    }

    /**
     * Create a new instance of Logger.
     * @param loggerID the id to identify one logger from another.
     * @return the newly created logger. 
     */
    public abstract Logger getLoggerInstance(String loggerID);

    /**
     * Add default channels to write the log messages to.
     * @param theLogChannels the channels to write the log messages to.
     * Of type LogChannel.
     */
    public void addDefaultLogChannels(Vector theLogChannels)
    {
        if (defaultLogChannels == null)
        {
            defaultLogChannels = new Vector();
        }

        CollectionHandler.addAll(defaultLogChannels, theLogChannels);
    }

    /**
     * Add default channels to write the debug messages to.
     * @param theLogChannels the channels to write the debug messages to.
     * Of type LogChannel.
     */
    public void addDefaultDebugChannels(Vector theLogChannels)
    {
        if (defaultDebugChannels == null)
        {
            defaultDebugChannels = new Vector();
        }

        CollectionHandler.addAll(defaultDebugChannels, theLogChannels);
    }

    /**
     * Get any default channels to write the log messages to.
     * @return a list of all default log channels.
     */
    protected Vector getDefaultLogChannels()
    {
        return defaultLogChannels;
    }

    /**
     * Get any default channels to write the debug messages to.
     * @return a list of all default debug channels.
     */
    protected Vector getDefaultDebugChannels()
    {
        return defaultDebugChannels;
    }

    /**
     * Create and store any new log channels associated with the new log id.
     * @param loggerID the id to identify one logger from another.
     * @return a list of all log channels associated with the logger.
     */
    protected abstract Vector getLogChannels(String loggerID);

    /**
     * Create and store any new debug channels associated with the new log id.
     * @param loggerID the id to identify one logger from another.
     * @return a list of all debug channels associated with the logger.
     */
    protected abstract Vector getDebugChannels(String loggerID);

    /**
     * Get the level to log messages to.
     * @return the value of logLevel.
     */
    public static String getLogLevel()
    {
        return logLevel;
    }

    /**
     * Get the level to debug messages to.
     * @return the value of debugLevel.
     */
    public static String getDebugLevel()
    {
        return debugLevel;
    }
}
